      SUBROUTINE AGCAP(I1,I2,I3,I4,I5,I6,I7,I8)
C     AGCAP PERFORMS AGC (AUTOMATIC GAIN CONTROL) USING THE VFC OF THE FPS AP.
C  AGC SCALES THE DATA SUCH THAT EACH OUTPUT WINDOW HAS THE SAME AVERAGE
C  ABSOLUTE AVERAGE.
C
C  ARGUMENTS:
C    I1 - IN     - THE AP ADDRESS OF THE INPUT ARRAY.
C    I2 - NADD   - THE NUMBER OF SAMPLES TO SKIP FROM IN BECAUSE THEY ARE DEAD.
C    I3 - WINLEN - THE NUMBER OF SAMPLES IN EACH WINDOW.
C    I4 - OUT    - THE AP ADDRESS OF THE OUTPUT ARRAY.
C    I5 - NOUT   - THE NUMBER OF VALUES TO OUTPUT (ALSO THE LENGTH OF THE INPUT).
C    I6 - PARAMS - THE AP ADDRESS OF 3 VARIABLES THAT MUST BE SET IN TH AP BY
C                  THE CALLING PROGRAM.
C                +0 - 1./WINLEN
C                +1 - A SMALL NUMBER TO PREVENT DIVIDE BY ZEROES
C                +2 - THE OUTPUT WINDOW LEVEL (AVERAGE ABSOLUTE VALUE)
C    I7 - SCR1   - THE AP ADDRESS OF A SCRATCH ARRAY NOUT LONG.
C    I8 - SCR2   - THE AP ADDRESS OF A SECOND SCRATCH ARRAY NOUT LONG.
C
C  WRITTEN AND COPYRIGHTED BY:
C  PAUL HENKART, SCRIPPS INSTITUTION OF OCEANOGRAPHY, 2 JANUARY 1980
C  ALL RIGHTS ARE RESERVED BY THE AUTHOR.  PERMISSION TO COPY OR REPRODUCE THIS
C  SUBROUTINE, BY COMPUTER OR OTHER MEANS, MAY BE OBTAINED ONLY FROM THE AUTHOR.
C
      IMPLICIT INTEGER*2(B-Z)
      IN=I1
      NADD=I2
      WINLEN=I3
      OUT=I4
      NOUT=I5
      PARAMS=I6
      SCR1=I7
      SCR2=I8
      WINL2=WINLEN/2
      N=NOUT-NADD
      N=N-WINLEN                                                          /* THE NUMBER OF WINDOWS IN THE TRACE
      ISTART=IN+NADD                                                    /* THE FIRST LIVE VALUE IN THE TRACE
      I=0
10    CALL SVEMG(ISTART,1,SCR1,WINLEN)                                    /* CALCULATE THE SUM OF THE MAGNITUDES
      I=I+1
      ISTART=ISTART+1
      SCR1=SCR1+1                                                        /* PUT THE NEXT RESULT IN THE NEXT AP LOCATION
      IF(I.LT.N) GO TO 10
      SCR1=I7                                                           /* THE BEGINNING OF THE SCRATCH ARRAY
      CALL VSMUL(SCR1,1,PARAMS,SCR1,1,N)                                /* DIVIDE THE BY N TO MAKE IT THE AVE
      IADD=PARAMS+1
      OLEVEL=PARAMS+2
      CALL VFILL(OLEVEL,SCR2,1,N)                                        /* MAKE A VECTOR OF THE OUTPUT LEVELS
      CALL VSADD(SCR1,1,IADD,SCR1,1,N)                                  /* MAKE SURE WE DON'T DIVIDE BY ZERO
      CALL VDIV(SCR1,1,SCR2,1,SCR1,1,N)                                 /* DIVIDE THE OLEVEL BY THE AVERAGE
      M=NADD+WINL2
      CALL VFILL(SCR1,SCR2,1,M)
      NEXT=SCR2+M
      CALL VMOV(SCR1,1,NEXT,1,N)                                        /* MOVE THE MULTIPLIERS
      LAST=SCR1+N
      LAST=LAST-1
      NEXT=NEXT+N
      CALL VFILL(LAST,NEXT,1,WINL2)                                      /* FILL THE LAST HALF OF THE LAST WINDOW
      CALL VMUL(SCR2,1,IN,1,OUT,1,NOUT)
      RETURN
      END
